﻿using AspectCore.DynamicProxy;
using HealthMonitor.Core.Common;

namespace HealthMonitor.Core.Pipeline.Aop.Update
{
    public class AssertValidUpdateValue : AopValueBase
    {
        public override async Task Invoke(CacheInterceptorContext context)
        {
            //方法需要存在参数用于生成缓存键和获取实体类型
            if (context.AopContext.Parameters.Length == 0 || context.AopContext.Parameters[0] == null)
            {
                throw new AssertValidException();
            }

            var param = context.AopContext.Parameters[0];
            var entityType = param.GetType();

            context.IsAsyncMethod = context.AopContext.IsAsync();
            context.EntityType = entityType;
            context.RequestId = context.AopContext.Parameters.Length > 1 ? context.AopContext.Parameters[1] + "" : "";

            using (var scope = context.Logger.BeginScope(new Dictionary<string, object> { ["RequestId"] = context.RequestId }))
            using (new CustomizeStopWatch($"修改aop[{entityType.Name}]", context.Logger))
            {
                await InvokeNextAsync(context);
            }
        }
    }
}
